@@ -284,14 +284,8 @@ impl<'a, 's> Context<'a, 's> {
         // Start with the public function symbols.
         let mut symbol_iter = global_symbols.iter();
         while let Some(symbol) = symbol_iter.next()? {
-            if let Ok(SymbolData::Public(PublicSymbol {
-                function,
-                name,
-                offset,
-                ..
-            })) = symbol.parse()
-            {
-                if function || is_executable_section(offset.section, sections) {
+            if let Ok(SymbolData::Public(PublicSymbol { name, offset, .. })) = symbol.parse() {
+                if is_executable_section(offset.section, sections) {
                     public_functions.push(PublicSymbolFunction {
                         start_offset: offset,
                         name,
@@ -306,7 +300,7 @@ impl<'a, 's> Context<'a, 's> {
         // Read the section contributions. This will let us find the right module
         // based on the PdbSectionInternalOffset that corresponds to the looked-up
         // address. This allows reading module info on demand.
-        let section_contributions = compute_section_contributions(debug_info)?;
+        let section_contributions = compute_section_contributions(debug_info, sections)?;
 
         Ok(Self {
             address_map,
@@ -617,8 +611,8 @@ impl<'a, 's> Context<'a, 's> {
             }
         };
 
-        let module_index = self.section_contributions[sc_index].module_index;
-        let basic_module_info = module_cache.get_basic_module_info(module_index);
+        let sc = &self.section_contributions[sc_index];
+        let basic_module_info = module_cache.get_basic_module_info(sc.module_index);
 
         if let Some(BasicModuleInfo {
             procedures,
@@ -640,7 +634,7 @@ impl<'a, 's> Context<'a, 's> {
             }) {
                 // Found a procedure at the requested offset.
                 return Some(PublicOrProcedureSymbol::Procedure(
-                    module_index,
+                    sc.module_index,
                     module_info,
                     &procedures[procedure_index],
                 ));
@@ -670,6 +664,10 @@ impl<'a, 's> Context<'a, 's> {
         if fun.start_offset.section != offset.section {
             return None;
         }
+        // Ignore symbols outside the section contribution.
+        if fun.start_offset.offset < sc.start_offset {
+            return None;
+        }
 
         Some(PublicOrProcedureSymbol::Public(fun))
     }
@@ -794,6 +792,30 @@ impl<'a, 's> BasicModuleInfo<'a, 's> {
                         type_index: proc.type_index,
                     });
                 }
+                Ok(SymbolData::SeparatedCode(data)) => {
+                    if data.len == 0 {
+                        continue;
+                    }
+
+                    // SeparatedCode references another procedure with data.parent_offset.
+                    // Usually the SeparatedCode symbol comes right after the referenced symbol.
+                    // Take the name and type_index from the referenced procedure.
+                    let (name, type_index) = match functions.last() {
+                        Some(proc) if proc.offset == data.parent_offset => {
+                            (proc.name, proc.type_index)
+                        }
+                        _ => continue,
+                    };
+
+                    functions.push(ProcedureSymbolFunction {
+                        offset: data.offset,
+                        len: data.len as u32,
+                        name,
+                        symbol_index: symbol.index(),
+                        end_symbol_index: data.end,
+                        type_index,
+                    });
+                }
                 Ok(SymbolData::Thunk(thunk)) => {
                     if thunk.len == 0 {
                         continue;
@@ -840,14 +862,14 @@ pub struct ModuleSectionContribution {
 /// and this function returns an error if any interleaving is detected.
 fn compute_section_contributions(
     debug_info: &DebugInformation<'_>,
+    sections: &[ImageSectionHeader],
 ) -> Result<Vec<ModuleSectionContribution>> {
-    let mut section_contribution_iter = debug_info.section_contributions()?;
+    let mut section_contribution_iter = debug_info
+        .section_contributions()?
+        .filter(|sc| Ok(sc.size != 0 && is_executable_section(sc.offset.section, sections)));
     let mut section_contributions = Vec::new();
 
-    while let Some(first_sc) = section_contribution_iter.next()? {
-        if first_sc.size == 0 {
-            continue;
-        }
+    if let Some(first_sc) = section_contribution_iter.next()? {
         let mut current_combined_sc = ModuleSectionContribution {
             section_index: first_sc.offset.section,
             start_offset: first_sc.offset.offset,
@@ -857,9 +879,6 @@ fn compute_section_contributions(
         // Assume that section contributions from the same section and module are
         // sorted and non-interleaved.
         while let Some(sc) = section_contribution_iter.next()? {
-            if sc.size == 0 {
-                continue;
-            }
             let section_index = sc.offset.section;
             let start_offset = sc.offset.offset;
             let end_offset = start_offset + sc.size;
@@ -912,14 +931,22 @@ fn compute_section_contributions(
     Ok(section_contributions)
 }
 
-fn is_executable_section(section_index: u16, sections: &[ImageSectionHeader]) -> bool {
-    let section = match sections.get(section_index as usize) {
-        Some(section) => section,
-        None => return false,
-    };
+/// section_index is a 1-based index from PdbInternalSectionOffset.
+fn get_section(section_index: u16, sections: &[ImageSectionHeader]) -> Option<&ImageSectionHeader> {
+    if section_index == 0 {
+        None
+    } else {
+        sections.get((section_index - 1) as usize)
+    }
+}
 
+/// section_index is a 1-based index from PdbInternalSectionOffset.
+fn is_executable_section(section_index: u16, sections: &[ImageSectionHeader]) -> bool {
     const IMAGE_SCN_MEM_EXECUTE: u32 = 0x20000000;
-    section.characteristics & IMAGE_SCN_MEM_EXECUTE != 0
+    match get_section(section_index, sections) {
+        Some(section) => section.characteristics & IMAGE_SCN_MEM_EXECUTE != 0,
+        None => false,
+    }
 }
 
 /// Offset and name of a function from a public symbol.
